/*-
 * Copyright 2015-2016 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Authors: Nils Goroll <nils.goroll@uplex.de>
 *          Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <string.h>

#include "vmod_convert.h"

#include "vrt.h"
#include "vas.h"

ssize_t
id_encode(const enum encoding enc, char *restrict const buf,
	  const size_t maxlen, const char *restrict const in,
	  const size_t inlen)
{
	(void) enc;
	AN(buf);

	if (maxlen < inlen + 1)
		return -1;
	if (in == NULL || inlen == 0)
		return 0;

	memcpy(buf, in, inlen);
	return inlen;
}

ssize_t
id_decode(const enum encoding enc, char *restrict const buf,
	  const size_t maxlen, const char *restrict const p, va_list ap)
{
	const char *next;
	char *dest = buf;
	size_t len, outlen = 0;

	(void) enc;
	AN(buf);

	if (p == vrt_magic_string_end)
		return 0;

	SKIP_EMPTY(next, ap);
	if (next == vrt_magic_string_end && (p == NULL || *p == '\0'))
		return 0;

	if (p != NULL && *p != '\0') {
		if ((len = strlen(p)) > maxlen)
			return -1;
		memcpy(buf, p, len);
		outlen = len;
		dest += len;
	}

	while (next != vrt_magic_string_end) {
		len = strlen(next);
		if ((outlen += len) > maxlen)
			return -1;
		memcpy(dest, next, len);
		dest += len;
		SKIP_EMPTY(next, ap);
	}

	return outlen;
}
